import sys

if sys.version_info >= (2, 7) and sys.version_info <= (3, 4):

    from django.test import TestCase, RequestFactory
    from jinjasql import JinjaSql

    import unittest

    @unittest.skipIf(sys.version_info < (2, 7), 
        "django version not supported in python 2.6.x")
    class DjangoTest(TestCase):
        def test_django_request_as_context(self):
            request = RequestFactory().get('/customer/details/?customer=1232&enterprise=9875')
            j = JinjaSql()
            query, bind_params = j.prepare_query("""select {{request.path}} 
                from dual 
                where customer={{request.GET.customer}}"""
                , {"request": request}
            )
            self.assertEqual(bind_params, [u"/customer/details/", u"1232"])


    def configure_django():
        from django.conf import settings
        import django

        settings.configure(
            DEBUG_PROPAGATE_EXCEPTIONS=True,
            DATABASES={'default': {'ENGINE': 'django.db.backends.sqlite3',
                                   'NAME': ':memory:'}},
            SITE_ID=1,
            SECRET_KEY='not very secret in tests',
            USE_I18N=True,
            USE_L10N=True,
            STATIC_URL='/static/',
            ROOT_URLCONF='tests.urls',
            TEMPLATE_LOADERS=(
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
            ),
            MIDDLEWARE_CLASSES=(
                'django.middleware.common.CommonMiddleware',
                'django.contrib.sessions.middleware.SessionMiddleware',
                'django.middleware.csrf.CsrfViewMiddleware',
                'django.contrib.auth.middleware.AuthenticationMiddleware',
                'django.contrib.messages.middleware.MessageMiddleware',
            ),
            INSTALLED_APPS=(
                'django.contrib.auth',
                'django.contrib.contenttypes',
                'django.contrib.sessions',
                'django.contrib.sites',
                'django.contrib.messages',
                'django.contrib.staticfiles',
            ),
            PASSWORD_HASHERS=(
                'django.contrib.auth.hashers.SHA1PasswordHasher',
                'django.contrib.auth.hashers.PBKDF2PasswordHasher',
                'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
                'django.contrib.auth.hashers.BCryptPasswordHasher',
                'django.contrib.auth.hashers.MD5PasswordHasher',
                'django.contrib.auth.hashers.CryptPasswordHasher',
            ),
        )
        django.setup()

    configure_django()



import os
from testcontainers.postgres import PostgresContainer
from testcontainers.mysql import MySqlContainer
import sqlalchemy
import unittest
from jinjasql import JinjaSql

class PostgresTest(unittest.TestCase):

    # Core idea inspired from 
    # https://stackoverflow.com/questions/8416208/in-python-is-there-a-good-idiom-for-using-context-managers-in-setup-teardown
    # 
    # Override the run method to automatically
    # a. launch a postgres docker container
    # b. create a sqlalchemy connection 
    # c. at the end of the test, kill the docker container
    def run(self, result=None):
        self.postgresql_test_container = PostgresContainer("postgres:15.3")
        if os.name == "nt":
            self.postgresql_test_container.get_container_host_ip = lambda: "localhost"
        self.postgresql_test_container.start()
        self.engine = sqlalchemy.create_engine(self.postgresql_test_container.get_connection_url(),echo=True)
        super(PostgresTest, self).run(result)

    def test_bind_array(self):
        'It should be possible to bind arrays in a query'
        j = JinjaSql(param_style='named')
        data = {
            "some_num": 1,
            "some_array": [1,2,3]
        }
        template = """
            SELECT {{some_num}} = ANY({{some_array}})
        """
        query, params = j.prepare_query(template, data)
        with self.engine.connect() as conn:
            result = conn.execute(sqlalchemy.text(query), params).fetchone()
        self.assertTrue(result[0])
    
    def test_quoted_tables(self):
        j = JinjaSql()
        data = {
            "all_tables": ("information_schema", "tables")
        }
        template = """
            select table_name from {{all_tables|identifier}}
            where table_name = 'pg_user'
        """
        query, params = j.prepare_query(template, data)
        with self.engine.connect() as conn:
            result = conn.execute(sqlalchemy.text(query), params).fetchall()
        self.assertEqual(len(result), 1)

class MySqlTest(unittest.TestCase):
    def run(self, result=None):
        self.container = (
            MySqlContainer("mysql/mysql-server", platform="linux/amd64")
            .with_exposed_ports(3306)
            .with_env("MYSQL_USER", "root")
            .with_env("MYSQL_PASSWORD", "test")
            .with_env("MYSQL_DATABASE", "test")
        )
        if os.name == "nt":
            self.container.get_container_host_ip = lambda: "localhost"
        self.container.start()
        self.engine = sqlalchemy.create_engine(self.container.get_connection_url())
        super(MySqlTest, self).run(result)

    def test_quoted_tables(self):
        j = JinjaSql(identifier_quote_character='`')
        data = {
            "database": "information_schema",
            "table_name" : "TABLES"
        }
        template = """
            select * from {{database|identifier}}.{{table_name|identifier}};
        """
        query, params = j.prepare_query(template, data)
        with self.engine.connect() as conn:
            result = conn.execute(sqlalchemy.text(query), params).fetchall()
        self.assertTrue(len(result)>1)

if __name__ == '__main__':
    unittest.main()


from __future__ import unicode_literals
import unittest
from jinja2 import DictLoader
from jinja2 import Environment
from jinjasql import JinjaSql
from jinjasql.core import InvalidBindParameterException
from datetime import date
from yaml import safe_load_all
from os.path import dirname, abspath, join


YAML_TESTS_ROOT = join(dirname(abspath(__file__)), "yaml")

_DATA = {
    "etc": {
        "columns": "project, timesheet, hours",
        "lt": "<",
        "gt": ">",
    },
    "ids": {
        "field1": "id",
        "field2": "name",
        "table": ("public", "users"),
    },
    "malicious": {
        "table": "users; drop table users; --",
        "field": "id\" FROM users; drop table users; --"
    },
    "request": {
        "project": {
            "id": 123,
            "name": "Acme Project"
        },
        "project_id": 123,
        "days": ["mon", "tue", "wed", "thu", "fri"],
        "day": "mon",
        "start_date": date.today(),
    },
    "session": {
        "user_id": u"sripathi"
    }
}

class JinjaSqlTest(unittest.TestCase):
    def setUp(self):
        self.j = JinjaSql(param_style="format")

    def test_import(self):
        utils = """
        {% macro print_where(value) -%}
        WHERE dummy_col = {{value}}
        {%- endmacro %}
        """
        source = """
        {% import 'utils.sql' as utils %}
        select * from dual {{ utils.print_where(100) }}
        """
        loader = DictLoader({"utils.sql" : utils})
        env = Environment(loader=loader)

        j = JinjaSql(env,param_style="format")
        query, bind_params = j.prepare_query(source, _DATA)
        expected_query = "select * from dual WHERE dummy_col = %s"
        self.assertEqual(query.strip(), expected_query.strip())
        self.assertEqual(len(bind_params), 1)
        self.assertEqual(list(bind_params)[0], 100)

    def test_include(self):
        where_clause = """where project_id = {{request.project_id}}"""
        
        source = """
        select * from dummy {% include 'where_clause.sql' %}
        """
        loader = DictLoader({"where_clause.sql" : where_clause})
        env = Environment(loader=loader)

        j = JinjaSql(env,param_style="format")
        query, bind_params = j.prepare_query(source, _DATA)
        expected_query = "select * from dummy where project_id = %s"
        self.assertEqual(query.strip(), expected_query.strip())
        self.assertEqual(len(bind_params), 1)
        self.assertEqual(list(bind_params)[0], 123)

    def test_precompiled_template(self):
        source = "select * from dummy where project_id = {{ request.project_id }}"
        j = JinjaSql(param_style="format")
        query, bind_params = j.prepare_query(j.env.from_string(source), _DATA)
        expected_query = "select * from dummy where project_id = %s"
        self.assertEqual(query.strip(), expected_query.strip())

    def test_large_inclause(self):
        num_of_params = 50000
        alphabets = ['A'] * num_of_params
        source = "SELECT 'x' WHERE 'A' in {{alphabets | inclause}}"
        j = JinjaSql(param_style="format")
        query, bind_params = j.prepare_query(source, {"alphabets": alphabets})
        self.assertEqual(len(bind_params), num_of_params)
        self.assertEqual(query, "SELECT 'x' WHERE 'A' in (" + "%s," * (num_of_params - 1) + "%s)")

    def test_large_likeclause(self):
        source = "SELECT 'x' WHERE project_id ilike {{('%%'~request.project_id~'%%')}}"
        j = JinjaSql(param_style="named")
        query, bind_params = j.prepare_query(source, _DATA)
        self.assertEqual(len(bind_params), 1)
        self.assertEqual(query, "SELECT 'x' WHERE project_id ilike :bind0_1")

    def test_identifier_filter(self):
        j = JinjaSql(param_style="format")
        template = 'select * from {{table_name | identifier}}'
        
        tests = [
            ('users', 'select * from "users"'),
            (('myschema', 'users'), 'select * from "myschema"."users"'),
            ('a"b', 'select * from "a""b"'),
            (('users',), 'select * from "users"'),
        ]
        for test in tests:
            query, _ = j.prepare_query(template, {'table_name': test[0]})
            self.assertEqual(query, test[1])


    def test_identifier_filter_backtick(self):
        j = JinjaSql(identifier_quote_character='`')
        template = 'select * from {{table_name | identifier}}'
        
        tests = [
            ('users', 'select * from `users`'),
            (('myschema', 'users'), 'select * from `myschema`.`users`'),
            ('a`b', 'select * from `a``b`'),
        ]
        for test in tests:
            query, _ = j.prepare_query(template, {'table_name': test[0]})
            self.assertEqual(query, test[1])

def generate_yaml_tests():
    file_path = join(YAML_TESTS_ROOT, "macros.yaml")
    with open(file_path) as f:
        configs = safe_load_all(f)
        for config in configs:
            yield (config['name'], _generate_test(config))

def _generate_test(config):
    def yaml_test(self):
        source = config['template']
        for (param_style, expected_sql) in config['expected_sql'].items():
            jinja = JinjaSql(param_style=param_style)
            query, bind_params = jinja.prepare_query(source, _DATA)

            if 'expected_params' in config:
                if param_style in ('pyformat', 'named'):
                    expected_params = config['expected_params']['as_dict']
                else:
                    expected_params = config['expected_params']['as_list']
                self.assertEqual(list(bind_params), expected_params)

            self.assertEqual(query.strip(), expected_sql.strip())

    return yaml_test

for test in generate_yaml_tests():
    test_name = test[0]
    test_function = test[1]
    setattr(JinjaSqlTest, test_name, test_function)

if __name__ == '__main__':
    unittest.main()
